﻿using System;
using System.Collections.Generic;
using System.Text;

using Com.Piwigo.Lib.DTO;
using Com.Piwigo.Lib.Proxy;
using Com.Piwigo.Lib.Proxy.Response;
using Com.Piwigo.Lib.DTO.Helper;
using Com.Piwigo.Lib.IService;

namespace Com.Piwigo.Lib.Service
{

    internal sealed class PwgSessionService : IPwgSessionService
    {
        private PwgSession PwgSession = null;

        /// <summary>
        /// Return informations about user logged
        /// </summary>
        /// <returns></returns>
        public PwgSession GetPwgSession()
        {
            if (PwgSession == null)
            {
                try
                {
                    PwgSessionProxyResponse response = PwgSessionProxy.pwg_session_getStatus();

                    if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                    {
                        if (response.Erreur != null)
                        {
                            throw new PwgServiceException("GetPwgSession, the server has return the error.", 
                                response.Erreur.Code,
                                response.Erreur.Message);
                        }
                        else
                        {
                            throw new PwgServiceException("GetPwgSession : a error occurs during server process.");
                        }
                    }
                    else
                    {
                        ConvertProxyResponseToDTO(response, ref PwgSession);
                    }
                } catch (PwgProxyException ex)
                {
                    throw new PwgServiceException("GetPwgSession : a error is raised by proxy.", ex);
                }

            }

            return PwgSession;
        }
        /// <summary>
        /// Log to the gallery using a account
        /// </summary>
        /// <param name="UserName"></param>
        /// <param name="Password"></param>
        /// <returns></returns>
        public PwgSession Login(String UserName, String Password)
        {
            PwgSession = null;
            Boolean isConnected = false;
            try
            {
                PwgSessionProxyResponse response = PwgSessionProxy.pwg_session_login(UserName, Password);

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok)
                {

                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("Login, the server has return the error.",
                            response.Erreur.Code,
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("Login : a error occurs during server process.");
                    }
                }
                else
                {
                    isConnected = true;
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("Login : a error is raised by proxy.", ex);
            }

            PwgSession sess= GetPwgSession();
            sess.IsConnected = isConnected;
            return sess;
        }

        public PwgSession Logout()
        {
            PwgSession = null;
            Boolean isConnected = false;
            try
            {
                PwgSessionProxyResponse response = PwgSessionProxy.pwg_session_logout();

                if (response.Retour != PwgBaseProxyReponseRetourEnum.Ok )
                {
                    if (response.Erreur != null)
                    {
                        throw new PwgServiceException("Logout, the server has return the error.",
                            response.Erreur.Code, 
                            response.Erreur.Message);
                    }
                    else
                    {
                        throw new PwgServiceException("Logout : a error occurs during server process.");
                    }
                }
            }
            catch (PwgProxyException ex)
            {
                throw new PwgServiceException("Logout : a error is raised by proxy.", ex);
            }

            PwgSession sess = GetPwgSession();
            sess.IsConnected = isConnected;
            return sess;
        }
        /// <summary>
        /// private: convert response to dto object
        /// </summary>
        /// <param name="response"></param>
        /// <param name="session"></param>
        internal static void ConvertProxyResponseToDTO(PwgSessionProxyResponse response, ref PwgSession session)
        {
            if (session == null)
            {
                session = new PwgSession();
            }

            if (response != null)
            {
                session.CharSet = response.CharSet;
                session.Language = response.Language;
                session.Status = PwgEnumHelper<PwgSessionStatusEnum>.enumValueOf(response.Status);
                session.Template = response.Template;
                session.Theme = response.Theme;
                session.UserName = response.UserName;
                session.SecurityToken = response.PwgToken;
            }

        }
    }
}
