﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Input;
using Com.Piwigo.Wpf.Helper;
using Com.Piwigo.Wpf.DTO;
using System.ComponentModel;

namespace Com.Piwigo.Wpf.Command
{
    public abstract class AsyncCommand : ICommand
    {
        public event EventHandler CanExecuteChanged;
        public event EventHandler RunWorkerStarting;
        public event RunWorkerCompletedEventHandler RunWorkerCompleted;

        public abstract string Text { get; }
        private bool _isRunning;
        public bool IsRunning
        {
            get { return _isRunning; }
            private set
            {
                _isRunning = value;
                if (CanExecuteChanged != null)
                    CanExecuteChanged(this, EventArgs.Empty);
                PwgCmdEvents.OnCmdRunningChanged(this, new EventArgs<bool>(_isRunning));
            }
        }

        protected void SendUiInfo(string strMessage)
        {
            PwgCmdEvents.OnCmdSendUiInfo(this, new EventArgs<string>(strMessage));
        }

        protected abstract void OnExecute(object parameter);

        public void Execute(object parameter)
        {
            try
            {
                onRunWorkerStarting();

                var worker = new BackgroundWorker();
                worker.DoWork += ((sender, e) => OnExecute(e.Argument));
                worker.RunWorkerCompleted += ((sender, e) => onRunWorkerCompleted(e));
                worker.RunWorkerAsync(parameter);
            }
            catch (Exception ex)
            {
                onRunWorkerCompleted(new RunWorkerCompletedEventArgs(null, ex, true));
            }
        }

        private void onRunWorkerStarting()
        {
            IsRunning = true;
            if (RunWorkerStarting != null)
                RunWorkerStarting(this, EventArgs.Empty);
        }

        private void onRunWorkerCompleted(RunWorkerCompletedEventArgs e)
        {
            IsRunning = false;
            if (RunWorkerCompleted != null)
                RunWorkerCompleted(this, e);
        }

        public virtual bool CanExecute(object parameter)
        {
            return !IsRunning;
        }
    }

}
